const appNameNice = 'Store Occupancy Manager';
const appNameParam = 'StoreOccupancyManager';
const appNameLowered = appNameParam.toLowerCase();

var appEnabled = 0;
var serviceArr;
var serviceArrTmp = [];
var scheduleTmpArr = [];
const MAX_ADDITIONAL_ITEMS = 6;
const MAX_COUNTER_ITEMS = 4;
const HTTP_OUTPUT_TYPES = {
  AXIS: "AXIS_OVERLAY",
  WS: "WS",
  API: "API"
};
const DEFAULT_HTTP_OUTPUT = {
  type: HTTP_OUTPUT_TYPES.AXIS,
  ws_token: "",
  api_token: "",
}

var alertID = 0;
var cameraImgUrl = '/axis-cgi/mjpg/video.cgi?fps=2';

var cameraResolutions = [];
var resolution = {};

var installPage = 'installApp.html';
var defaultPage = 'settings.html';

var params;
var nbrOfConfigs;
var appVersion;
var serverUrl;
var serialNumber;
var cameraModel;
var cameraFirmware;
var cameraArchitecture;
var secondaryDNS;
const DEFAULT_COUNTER_CORRECTION = {
  "correction": 0,
  "reset_time": "",
}
let counterCorrection;

var licenseState = '';
var licenseActivated = false;
var licenseExpiration = '';

var previewRequests = [];

let imageManager = ImageManager(appNameLowered);

$(document).ready(async function() {

  $.ajaxSetup({
    cache: false // Disable caching of AJAX responses
  });

  params = await getParameter('Image.NbrOfConfigs');
  nbrOfConfigs = parseInt(params['image.nbrofconfigs']);
  var paramList =
    appNameParam + '.Enabled,' +
    appNameParam + '.Configuration,' +
    appNameParam + '.Version,' +
    appNameParam + '.ServerUrl,' +
    appNameParam + '.CounterCorrection,' +
    'Properties.System.SerialNumber,' +
    'Properties.System.Architecture,' +
    'Properties.Firmware.Version,' +
    'Properties.Image.Resolution,' +
    'Image.I0.Appearance.Resolution,' +
    'Network.DNSServer2,' +
    'Brand.ProdNbr';
  // Add image props to paramList
  for(var i = 0; i < nbrOfConfigs; i++) {
    paramList += ',Properties.Image.I' + i + '.Resolution,' + 'Image.I' + i + '.Appearance.Resolution,Image.I' + i + '.Name,Image.I' + i + '.Source,Image.I' + i + '.Enabled';
  }

  params = await getParameter(paramList);
  appEnabled = parseInt(params[appNameLowered + '.enabled']);
  serviceArr = params[appNameLowered + '.configuration'] || '{}';
  appVersion = params[appNameLowered + '.version'];
  serverUrl = params[appNameLowered + '.serverurl'];
  counterCorrection = Object.assign(DEFAULT_COUNTER_CORRECTION, JSON.parse(params[appNameLowered + '.countercorrection'] || '{}'));
  serialNumber = params['properties.system.serialnumber'];
  cameraModel = params['brand.prodnbr'];
  cameraFirmware = params['properties.firmware.version'];
  cameraArchitecture = params['properties.system.architecture'];
  secondaryDNS = params['network.dnsserver2'];

  // Show version
  $('#versionNumber').text(appVersion).show();

  // Replace server links
  replaceServerUrl(serverUrl);

  // Start acap application
  await startAcapApp();

  // Generate service component
  serviceArrTmp = Object.assign(createDefaultServiceArr(), JSON.parse(serviceArr));

  // Check license
  const licenseResult = await isLicenseValid();
  licenseState = licenseResult[0];
  licenseActivated = licenseResult[0] === 'Valid';
  licenseExpiration = licenseResult[1];
  if (!licenseActivated) {
    showExpiredLicense();
  }

  let selectedCamera = 0;
  if(serviceArrTmp.camera_list.length > 0) {
    selectedCamera = serviceArrTmp.camera_list[0];
  }
  // Available resolutions for all video channels
  setNewResolutionList(params['properties.image.i' + selectedCamera + '.resolution']);
  // Default resolution for all video channels
  setNewDefaultResolution(params['image.i' + selectedCamera + '.appearance.resolution']);

  // Disable text overlay
  $.get('/axis-cgi/param.cgi?action=update&Image.I0.Text.TextEnabled=no');

  // Set secondary DNS server
  if (secondaryDNS.indexOf('0.0.0.0') != -1) {
    $.get('/axis-cgi/param.cgi?action=update&Network.DNSServer2=8.8.8.8');
  }

  // Is suported from fw 7.10 or 9.10 on armv7hf
  var fwSupported = [7, 10];
  if (isArmv7hf()) fwSupported = [9, 10];
  var fw = cameraFirmware.split('.');
  for (var i = 0; i < fwSupported.length && i < fw.length; i++) {
    if (parseInt(fw[i]) < fwSupported[i]) {
      showMessage('danger', appNameNice + ' is compatible with camera firmware ' + fwSupported.join('.') + ' or later. Your camera is using ' + cameraFirmware + ' now. Please check you have the latest firmware on your camera.', true);
      break;
    } else if (parseInt(fw[i]) > fwSupported[i]) {
      break;
    }
  }

  // Check latest version
  try {
    let latestACAPVersion = await getLatestACAPVersion(appNameLowered, cameraModel, serverUrl);
    let lastVersion = latestACAPVersion.split('.'); //format ['1', '0', '6']
    let currentVersion = appVersion.split('.');
    for (var i = 0; i < lastVersion.length && i < currentVersion.length; i++) {
      if (parseInt(currentVersion[i]) < parseInt(lastVersion[i])) {
        showMessage('info', 'New ' + appNameNice + ' ' + latestACAPVersion + ' is available. <a href="#" class="btn btn-light" onclick="prepareUpdate(appNameLowered, appNameNice, serverUrl, \'' + latestACAPVersion + '\', \'' + appVersion + '\')">Update now</a>', true);
        break;
      } else if (parseInt(currentVersion[i]) > parseInt(lastVersion[i])) {
        break;
      }
    }
  } catch(error) {
    console.log(error);
  }


  // Fill form with values
  await initForm();

  // Register events on form elements
  registerEvents();

  // Check connection on counterCamera
  const connectionResult = await checkConnection();
  collapseCounterGraphicsSettingsContent(connectionResult);

  // Check number of running graphics on camera
  checkGraphicsCount();
});

async function initForm() {
  $('#enabled').val(appEnabled ? "Yes" : "No");
  const videoChannelId = '#videoChannel';
  await createVideoChannelElement(videoChannelId, serviceArrTmp.camera_list);
  $(videoChannelId).hideSpinner();

  createResolutionList('#resolution');

  createCameraPreview($(videoChannelId), resolution.width, resolution.height, serviceArrTmp.camera_list);

  $('#maxInStoreLimit').val(serviceArrTmp.max_in_store_limit);

  serviceArrTmp.output = Object.assign(DEFAULT_HTTP_OUTPUT, serviceArrTmp.output);
  let httpOutput = serviceArrTmp.output;
  $('#outputType').val(httpOutput.type);
  let showUrl = httpOutput.type === HTTP_OUTPUT_TYPES.WS || httpOutput.type === HTTP_OUTPUT_TYPES.API;
  $('.outputTypeUrlRow').toggle(showUrl);
  setWsToken();
  setApiToken();
  setOutputTypeUrlInput();

  $('#counterCorrection').val(counterCorrection.correction);
  $('#counterCorrectionReset').val(counterCorrection.reset_time);

  $('#counterCameraHost').val(serviceArrTmp.counter_camera_host);
  $('#counterCameraPort').val(serviceArrTmp.counter_camera_port);
  $('#counterCameraUser').val(serviceArrTmp.counter_camera_user);
  $('#counterCameraPass').val(serviceArrTmp.counter_camera_pass);

  createApcPanel();
  fillApcPanel();

  $('#counterCameraType').val(serviceArrTmp.counter_camera_type || "aoe");
  setCameraTypePanel();

  fillCounterGraphicsSettings();
  createAdditionalGraphicsPanel();
  fillAdditionalGraphicsPanel();
  fillInfotickerSettings();


  let eventThreshold = new Thresholds(serviceArrTmp.events);
  eventThreshold.init();

  let alarmThreshold = new Alarms(serviceArrTmp.events);
  alarmThreshold.init();
}

function fillCounterGraphicsSettings() {
  $('#counterGraphicsEnabled').val(serviceArrTmp.counter_graphics.enabled ? "Yes" : "No");
  setServiceStatus('#checkCounterGraphicsStatus', $('#counterGraphicsEnabled').val());
  createPositionList('#counterGraphicsPosition', serviceArrTmp.counter_graphics.coordSystem);
  $('#counterGraphicsOffsetX').val(serviceArrTmp.counter_graphics.pos_x);
  $('#counterGraphicsOffsetY').val(serviceArrTmp.counter_graphics.pos_y);
  $('#counterGraphicsScale').val(convertScaleFloatToPercentage(serviceArrTmp.counter_graphics.scale));
  createImageListInOption('#counterGraphicsImageEnter', true, 'Default', serviceArrTmp.counter_graphics.image_enter);
  createImageListInOption('#counterGraphicsImageStop', true, 'Default', serviceArrTmp.counter_graphics.image_stop);
  $('#counterGraphicsLabelMaxLimit').val(serviceArrTmp.counter_graphics.label_max_limit);
  $('#counterGraphicsLabelInStore').val(serviceArrTmp.counter_graphics.label_in_store);
  $('#counterGraphicsLabelMayEnter').val(serviceArrTmp.counter_graphics.label_may_enter);
  $('#counterGraphicsFullscreen').val(serviceArrTmp.counter_graphics.fullscreen ? "Yes" : "No");
  if (!isMips()) {
    $('.counterGraphicsFullscreenRow').show();
  } else {
    $('.counterGraphicsFullscreenRow').hide();
  }
}

function setWsToken() {
  if (serviceArrTmp.output.ws_token === "") {
    serviceArrTmp.output.ws_token = generateToken();
  }
}

function setApiToken() {
  if (serviceArrTmp.output.api_token === "") {
    serviceArrTmp.output.api_token = generateToken();
  }
}

function setOutputTypeUrlInput() {
  let isWS = serviceArrTmp.output.type === HTTP_OUTPUT_TYPES.WS;
  let token = isWS ? serviceArrTmp.output.ws_token : serviceArrTmp.output.api_token;
  let template = isWS ? 'output.html' : 'output_slow.html';

  let outputUrl = createOutputUrl(template, token);
  $('#outputTypeUrl').val(outputUrl);
}

function generateToken() {
  let array = new Uint32Array(4);
  let crypto = window.crypto.getRandomValues(array);
  return crypto.join("")
}

function toggleOutputTypeUrl(e) {
  const typeRow = $('.outputTypeUrlRow');
  let outputTypeValue = $(e.target).val();
  let isTypeUrl = outputTypeValue === HTTP_OUTPUT_TYPES.WS || outputTypeValue === HTTP_OUTPUT_TYPES.API;
  if (isTypeUrl) {
    setWsToken();
    setApiToken();
    setOutputTypeUrlInput();
  }
  typeRow.toggle(isTypeUrl);
}

function createOutputUrl(template, token) {
  let url = new URL('/local/storeoccupancymanager/' + template, window.location.origin);
  let urlParams = new URLSearchParams(url.search);
  urlParams.set('token', token);
  url.search = urlParams;
  return url;
}

function fillInfotickerSettings() {
  if (!isMips()) {
    $('.infotickerCrawlLeft').show();
  } else {
    $('.infotickerCrawlLeft').hide();
  }
  $('#infotickerEnabled').val( serviceArrTmp.infoticker.enabled ? "Yes" : "No");
  setServiceStatus('#checkInfotickerStatus', $('#infotickerEnabled').val());
  $('#infotickerShowClock').val(serviceArrTmp.infoticker.showClock ? "Yes" : "No");
  if (serviceArrTmp.infoticker.clockType !== undefined) {
    $('#infotickerRadioClock').val(serviceArrTmp.infoticker.clockType);
  }
  if (serviceArrTmp.infoticker.textColor !== "" && serviceArrTmp.infoticker.textColor !== undefined) {
    var textColor = rgbToHex(serviceArrTmp.infoticker.textColor);
    $('#infotickerTextColor').val(textColor);
    $('#colorPaletteForTextColor').val(textColor);
  }
  if (serviceArrTmp.infoticker.bgColor !== "" && serviceArrTmp.infoticker.bgColor !== undefined) {
    var bgColor = rgbToHex(serviceArrTmp.infoticker.bgColor);
    $('#infotickerBgColor').val(bgColor);
    $('#colorPaletteForBgColor').val(bgColor);
  }
  if (serviceArrTmp.infoticker.sourceType === "") {
    $('#infotickerSourceType').val('text');
  } else {
    $('#infotickerSourceType').val(serviceArrTmp.infoticker.sourceType);
  }
  if (serviceArrTmp.infoticker.sourceType === 'text') {
    $('#infotickerSource-text').show();
  }

  $('#infotickerSource').val(serviceArrTmp.infoticker.source);
  $('#infotickerFontSize').val(parseInt(serviceArrTmp.infoticker.fontSize));
  $('#infotickerSwitchingTime').val(serviceArrTmp.infoticker.switchingTime);

  if (!isMips()) {
    //if yes, disable switchingTime and NumberOfLines
    if (!serviceArrTmp.infoticker.crawlLeft) {
      $('#infotickerCrawlLeft').val("No");
      $('#infotickerSwitchingTime').prop('disabled', false);
      $('#infotickerNumberOfLines').prop('disabled', false);
    } else {
      $('#infotickerSwitchingTime').prop('disabled', true);
      $('#infotickerNumberOfLines')
        .val(1)
        .prop('disabled', true);
      if(serviceArrTmp.infoticker.crawlSpeed === 1) $('#infotickerCrawlLeft').val("Slow");
      if(serviceArrTmp.infoticker.crawlSpeed === 2) $('#infotickerCrawlLeft').val("Medium");
      if(serviceArrTmp.infoticker.crawlSpeed === 4) $('#infotickerCrawlLeft').val("Fast");
    }
  }
  $('#infotickerNumberOfLines').val(serviceArrTmp.infoticker.numberOfLines);
}

function getScheduleFromServiceArrTmp(target) {
  if(target === 'counter_graphics') {
    return serviceArrTmp.counter_graphics.schedule;
  } else if (target === 'infoticker') {
    return serviceArrTmp.infoticker.schedule;
  } else if (target.indexOf('additional_graphics') > -1) {
    var targetName = target.split('_');
    var additionalGraphicsNumber = targetName[targetName.length - 1];
    return serviceArrTmp.additional_graphics[additionalGraphicsNumber - 1].schedule;
  }
}

function toggleFilledScheduleIcon() {
  let counterGraphicsScheduleBtnIcon = $('.scheduleBtn[data-schedule-target="counter_graphics"] .scheduleFilled');
  let infotickerScheduleBtnIcon = $('.scheduleBtn[data-schedule-target="infoticker"] .scheduleFilled');
  if (serviceArrTmp.counter_graphics.schedule !== '') {
    counterGraphicsScheduleBtnIcon.show();
  } else {
    counterGraphicsScheduleBtnIcon.hide();
  }
  if (serviceArrTmp.infoticker.schedule !== '') {
    infotickerScheduleBtnIcon.show();
  } else {
    infotickerScheduleBtnIcon.hide();
  }
  for(var i = 1; i <= serviceArrTmp.additional_graphics.length; i++) {
    var additionalGraphicsItemScheduleBtnIcon = $('.scheduleBtn[data-schedule-target="additional_graphics_' + i + '"] .scheduleFilled');
    if (serviceArrTmp.additional_graphics[i-1].schedule !== '') {
      additionalGraphicsItemScheduleBtnIcon.show();
    } else {
      additionalGraphicsItemScheduleBtnIcon.hide();
    }
  }
}

function setScheduleToServiceArrTmp(target, schedule) {
  if(target === 'counter_graphics') {
    serviceArrTmp.counter_graphics.schedule = schedule;
  } else if (target === 'infoticker') {
    serviceArrTmp.infoticker.schedule = schedule;
  } else if (target.indexOf('additional_graphics') > -1) {
    var targetName = target.split('_');
    var additionalGraphicsNumber = targetName[targetName.length - 1];
    serviceArrTmp.additional_graphics[additionalGraphicsNumber - 1].schedule = schedule;
  }
  toggleFilledScheduleIcon();
}

async function scheduleClickedCallback(target) {
  var scheduleArr = await getScheduleFromServiceArrTmp(target);
  $("#scheduleModal").modal({show: true});
  $('#scheduleSaveBtn').attr('data-schedule-target', target);
  scheduleTmpArr = [];
  if (scheduleArr != undefined) {
    scheduleTmpArr = scheduleArr.split('|');
  }
  createScheduleTable(scheduleTmpArr);
}

function createScheduleTable(scheduleArr) {
  var table = '<tbody>';
  if (scheduleArr.length) {
    for (var i = 0; i < scheduleArr.length; i++) {
      var interval = scheduleArr[i].trim();
      if (interval.length == 0) {
        scheduleTmpArr = [];
        createScheduleTable(scheduleTmpArr);
        return;
      }
      var intervalArr = interval.split('..');
      table += '<tr class="schedule" data-schedule="' + interval + '">';
      table += '<td>' + intToDay(parseInt(intervalArr[0][0])) + ' ' + intervalArr[0].substring(2) + '</td>';
      table += '<td> - </td>';
      table += '<td>' + intToDay(parseInt(intervalArr[1][0])) + ' ' + intervalArr[1].substring(2) + '</td>';
      table += '<td><button id="removeSchedule' + i + '" class="btn btn-sm btn-primary removeSchedule">Remove</button></td>';
      table += '</tr>';
    }
  }
  else
    table += '<tr class="schedule"><td colspan="4">No schedule set.</td></tr>';

  table += '</tbody>';

  $('#scheduleList').html(table);
  $('.removeSchedule').click(removeScheduleClickedCallback);
}

function scheduleFromDayChangeCallback() {
  $('#toDay').val($('#fromDay').val());
}

function scheduleAddClickedCallback() {
  var fromDay = $('#fromDay')[0].selectedIndex + 1;
  var fromTime = $('#fromTime').val();
  var toDay = $('#toDay')[0].selectedIndex + 1;
  var toTime = $('#toTime').val();
  scheduleTmpArr.push(fromDay + ' ' + validateTime(fromTime, '00:00:00') + '..' + toDay + ' ' + validateTime(toTime, '23:59:59'));
  createScheduleTable(scheduleTmpArr);
  return false; // Prevent form submission
}

function removeScheduleClickedCallback(event) {
  var targ = event.target || event.srcElement;
  var itemPos = targ.id.substring(14);
  scheduleTmpArr.splice(itemPos, 1);
  createScheduleTable(scheduleTmpArr);
}

function scheduleSaveClickedCallback(target) {
  var schedule = '';
  for (var i = 0; i < scheduleTmpArr.length; i++) {
    schedule += scheduleTmpArr[i];
    if (i < scheduleTmpArr.length - 1)
      schedule += '|';
  }
  if (scheduleTmpArr.length > 0) { //enable saveBtn
    enableSaveBtn();
  }
  setScheduleToServiceArrTmp(target, schedule);
  showSaveWarning();
}

function validateTime(inputTime, defaultVal) {
  var tArr = inputTime.split(':');
  var tArrValidated = [];
  for (var i = 0; i < 3; i++) {
    if (i < tArr.length) {
      var num = parseInt(tArr[i]);
      if (!Number.isNaN(num)) {
        tArrValidated.push(('0' + num).slice(-2));
      } else {
        console.log('Time conversion error: ' + tArr + ', default value is used: ' + defaultVal);
        tArrValidated = defaultVal.split(':');
        break;
      }
    } else {
      tArrValidated.push('00');
    }
  }
  return tArrValidated.join(':');
}

function intToDay(dayNum) {
  switch (dayNum) {
    case 1:
      return 'Mon';
    case 2:
      return 'Tue';
    case 3:
      return 'Wed';
    case 4:
      return 'Thu';
    case 5:
      return 'Fri';
    case 6:
      return 'Sat';
    case 7:
      return 'Sun';
    default:
      return '';
  }
}

function collapseCounterGraphicsSettingsContent(collapse) {
  if (collapse) {
    $('#counterGraphicsSettingsContent').collapse('hide');
  } else {
    $('#counterGraphicsSettingsContent').collapse('show');
  }
}

function checkGraphicsCount() {
  let sumOfEnabledGraphicsWithNoEntry = enabledInfotickerCount() + enabledCounterGraphicsCount() + enabledAdditionalGraphicsAlwaysCount() + enabledAdditionalGraphicsNoEntry();
  let sumOfEnabledGraphicsWithEntryAllowed = enabledInfotickerCount() + enabledCounterGraphicsCount() + enabledAdditionalGraphicsAlwaysCount() + enabledAdditionalGraphicsEntryAllowed();

  if(!isScheduleFilled()) { //ignore enabled services count if schedule is filled
    if(!isMips()) {
      if(sumOfEnabledGraphicsWithNoEntry > 4 || sumOfEnabledGraphicsWithEntryAllowed > 4) {
        disableSaveBtn();
        showMessage('warning', 'A maximum of four graphics can be display in total.', false, 'maxFourGraphics');
      } else {
        enableSaveBtn();
      }
    } else {
      if(sumOfEnabledGraphicsWithNoEntry > 2 || sumOfEnabledGraphicsWithEntryAllowed > 2) {
        disableSaveBtn();
        showMessage('warning', 'A maximum of two graphics can be display in total.', false, 'maxTwoGraphics');
      } else {
        enableSaveBtn();
      }
    }
  } else {
    enableSaveBtn();
  }
}

function isScheduleFilled() {
  if (serviceArrTmp.counter_graphics.schedule !== '') return true;
  if (serviceArrTmp.infoticker.schedule !== '') return true;
  for(var i = 0; i < serviceArrTmp.additional_graphics.length - 1; i++) {
    if (serviceArrTmp.additional_graphics[i].schedule !== '') return true;
  }
  return false;
}

function disableSaveBtn(){
  $('#saveBtn').prop('disabled', true);
}

function enableSaveBtn(){
  $('#saveBtn').prop('disabled', false);
}

function enabledInfotickerCount() {
  return serviceArrTmp.infoticker.enabled ? 1 : 0;
}

function enabledCounterGraphicsCount() {
  return serviceArrTmp.counter_graphics.enabled ? 1 : 0;
}

function enabledAdditionalGraphicsAlwaysCount() {
  let count = 0;
  for(var i = 1; i <= MAX_ADDITIONAL_ITEMS; i++) {
    if(serviceArrTmp.additional_graphics[i - 1].image !== "" && serviceArrTmp.additional_graphics[i - 1].visibility === "always") {
      count++;
    }
  }
  return count;
}

function enabledAdditionalGraphicsNoEntry() {
  let count = 0;
  for(var i = 1; i <= MAX_ADDITIONAL_ITEMS; i++) {
    if(serviceArrTmp.additional_graphics[i - 1].image !== "" && serviceArrTmp.additional_graphics[i - 1].visibility === "stop") {
      count++;
    }
  }
  return count;
}

function enabledAdditionalGraphicsEntryAllowed() {
  let count = 0;
  for(var i = 1; i <= MAX_ADDITIONAL_ITEMS; i++) {
    if(serviceArrTmp.additional_graphics[i - 1].image !== "" && serviceArrTmp.additional_graphics[i - 1].visibility === "enter") {
      count++;
    }
  }
  return count;
}

function createDefaultServiceArr() {
  return {
      "counter_camera_host": "127.0.0.1",
      "counter_camera_port": 80,
      "counter_camera_user": "root",
      "counter_camera_pass": "",
      "counter_camera_type": "aoe", // apc/aoe/xovis/axis3d
      "counter_camera_list": [
        {
          "host": "127.0.0.1",
          "port": 80,
          "user": "root",
          "pass": "",
          "element_name": "",
        },{
          "host": "127.0.0.1",
          "port": 80,
          "user": "root",
          "pass": "",
          "element_name": "",
        },{
          "host": "127.0.0.1",
          "port": 80,
          "user": "root",
          "pass": "",
          "element_name": "",
        },{
          "host": "127.0.0.1",
          "port": 80,
          "user": "root",
          "pass": "",
          "element_name": "",
        }
      ],
      "max_in_store_limit": 10,
      "stream_width": 1920,
      "stream_height": 1080,
      "camera_list": [0],
      "output": {
        "type": "API",//HTTP_OUTPUT_TYPES.AXIS, // WS/AXIS_OVERLAY
        "ws_token": "",
        "api_token": "",
      },
      "counter_graphics": {
        "enabled": true,
        "coordSystem": "top_left",
        "pos_x": 10,
        "pos_y": 10,
        "scale": 1.0,
        "image_enter": "",
        "image_stop": "",
        "label_max_limit": "",
        "label_in_store": "",
        "label_may_enter": "",
        "fullscreen": false,
        "schedule": "",
      },
      "additional_graphics": [
        {
          "visibility": "always",
          "coordSystem": "top_left",
          "pos_x": 10,
          "pos_y": 10,
          "scale": 1.0,
          "image": "",
          "schedule": ""
        },{
          "visibility": "always",
          "coordSystem": "top_left",
          "pos_x": 10,
          "pos_y": 10,
          "scale": 1.0,
          "image": "",
          "schedule": ""
        },{
          "visibility": "always",
          "coordSystem": "top_left",
          "pos_x": 10,
          "pos_y": 10,
          "scale": 1.0,
          "image": "",
          "schedule": ""
        },{
          "visibility": "always",
          "coordSystem": "top_left",
          "pos_x": 10,
          "pos_y": 10,
          "scale": 1.0,
          "image": "",
          "schedule": ""
        },{
          "visibility": "always",
          "coordSystem": "top_left",
          "pos_x": 10,
          "pos_y": 10,
          "scale": 1.0,
          "image": "",
          "schedule": ""
        },{
          "visibility": "always",
          "coordSystem": "top_left",
          "pos_x": 10,
          "pos_y": 10,
          "scale": 1.0,
          "image": "",
          "schedule": ""
        }
      ],
      "infoticker": {
        "enabled": false,
        "showClock": 1,
        "clockType": "24h",
        "textColor": "0,0,0,1",
        "bgColor": "255,255,255,1",
        "numberOfLines": 1,
        "switchingTime": 5,
        "crawlLeft": false,
        "crawlSpeed": 1.0,
        "fontSize": 40,
        "sourceType": "", // text,url
        "source": "",
        "schedule": ""
      },
      "events": [
        {
          "id": 1,
          "type": "threshold",
          "max_in_store_limit": -1
        },
        {
          "id": 2,
          "type": "alarm",
          "max_in_store_limit": -1
        }
      ],
    }
}

/**
 * Set default value for camera user input
 * @param {jQuery} cameraUserJqueryElement - jQuery object
 * @param {String} cameraTypeValue - type of counter ("aoe","apc","xovis","axis3d")
 */
function setDefaultCameraUserValue(cameraUserJqueryElement, cameraTypeValue) {
  if (cameraTypeValue === 'xovis') { // Xovis has default value: admin
    if (cameraUserJqueryElement.val() === 'root') {
      cameraUserJqueryElement.val('admin');
    }
  } else { // Axis Occupancy Estimator and Axis People Counter has default value: root
    if (cameraUserJqueryElement.val() === 'admin') {
      cameraUserJqueryElement.val('root');
    }
  }
}

function setCameraTypePanel() {
  $('#apcPanel, #aoePanel').hide();
  let cameraTypeValue = $('#counterCameraType').val();
  if (cameraTypeValue !== 'aoe' && cameraTypeValue !== 'axis3d' ) {
    $('#apcPanelInputs .cameraUserDefaultValue').each((index, element) => setDefaultCameraUserValue($(element), cameraTypeValue));
  }
  if (cameraTypeValue === 'xovis') {
    cameraTypeValue = 'apc';
    $('.xovisElementName').show();
  } else {
    $('.xovisElementName').hide();
  }
  let z;
  if (cameraTypeValue === 'axis3d'){
    z = 'aoe';
  } else {
    z = cameraTypeValue;
  }
  $('#' + z + 'Panel').show();
  checkConnection();
}

function registerEvents() {
  $('.form-control').unbind().change(formChangedCallback);
  $('.form-control-inline').unbind().change(formChangedCallback);
  $('.triggerConnectionCheck').change(checkConnection);
  $('#counterCameraType').unbind().change(function() {
    setCameraTypePanel();
    formChangedCallback();
  });
  $('[title]').tooltip();
  $("#saveBtn").unbind().click(async function() {
    let counterCorrectionCorrection = null;
    let value = $('#counterCorrectionOccupancy').val();
    if (value !== "") { // Only if "Actualy in" wil be filled
      try {
        counterCorrectionCorrection = await getCounterCorrection(value);
      } catch (e) {
        showMessage('error', e.message);
      }
      $('#counterCorrection').val(counterCorrectionCorrection);
      $('#counterCorrectionOccupancy').val("");
      counterCorrection.correction = parseInt(counterCorrectionCorrection);
    }

    saveBtnCallback();
  });
  $("#discardOkBtn").unbind().click(discardBtnCallback);
  $('#licenseInfoBtn').unbind().click(showLicenseInfoCallback);
  $('#imageManagerBtn').unbind().click(showImageManagerCallback);
  // Change image select after close modal
  $('body').off('hide.bs.modal').on('hide.bs.modal', '#fileManagerModal', function() {
    refreshSelectWithImageOptions();
  });
  $('.videoChannel').unbind().change(function() {
    serviceArrTmp.camera_list = saveCameraList();
    let selectedCamera = 0;
    if(serviceArrTmp.camera_list.length > 0) {
      selectedCamera = serviceArrTmp.camera_list[0];
    }
    setNewResolutionList(params['properties.image.i' + selectedCamera + '.resolution']);
    setNewDefaultResolution(params['image.i' + selectedCamera  + '.appearance.resolution']);
    createResolutionList('#resolution');
    formChangedCallback();
  });
  $('#outputType').change(toggleOutputTypeUrl);
  $('#outputTypeUrl').click(function() {
    copyValue(this, 'Web URL was copied to clipboard.');
  });
  $('.copyTypeUrl').click(function() {
    copyValue($('#outputTypeUrl')[0], 'Web URL was copied to clipboard.');
  });
  $('#counterCorrectionReset').attachValidation({
    pattern: '^$|^[0-9]{2}:[0-9]{2}$',
    invalidMsg: 'Only format HH:MM is supported.',
    disableBtnId: 'saveBtn'
  });
  $('#counterGraphicsEnabled').change( function() {
    setServiceStatus('#checkCounterGraphicsStatus', $(this).val());
  });
  $('#additionalGraphicsOverlaySettingsContent .form-control').change( function() {
    setAdditionalGraphicsOverlayStatus();
  });
  $('.scaleRangeValidation').on('keyup change', function() {
    validateRange($(this), {
      minVal: 30,
      maxVal: 200,
      invalidMsg: 'Must be from interval 30-200%.',
      disableBtnId: 'saveBtn'
    });
  });
  // Infoticker events
  $('#infotickerEnabled').change( function() {
    setServiceStatus('#checkInfotickerStatus', $(this).val());
  });

  // Schedule events
  $('#scheduleAddBtn').click(scheduleAddClickedCallback);
  $('#scheduleSaveBtn').click(function() {
    scheduleSaveClickedCallback($(this).attr('data-schedule-target'));
  });
  $('#fromDay').unbind().click(scheduleFromDayChangeCallback);
  $('#fromDay').unbind().change(scheduleFromDayChangeCallback);
  $('.scheduleBtn').unbind().click( function() {
    var target = $(this).attr('data-schedule-target');
    scheduleClickedCallback(target);
  });
  toggleFilledScheduleIcon();

  if (!isMips()) {
    $('.infotickerCrawlLeft').show();
    $('#infotickerCrawlLeft').change( function () {
      if ($(this).val() === "No") {
        $('#infotickerSwitchingTime').prop('disabled', false);
        $('#infotickerNumberOfLines').prop('disabled', false);
      } else {
        $('#infotickerSwitchingTime').prop('disabled', true);
        $('#infotickerNumberOfLines')
          .val(1)
          .prop('disabled', true);
      }
      formChangedCallback();
    });
  } else {
    $('.infotickerCrawlLeft').hide();
  }
  $('#infotickerSourceType').change( function() {
    if ($(this).val() === 'text') {
      $('#infotickerSource-text').show();
      $('#infotickerSource').attr('placeholder','Use Enter to break the line');
    } else {
      $('#infotickerSource-text').hide();
      $('#infotickerSource').attr('placeholder','Use UTF coding in TXT file.');
    }
  });
  $('#colorPaletteForTextColor').change( function() {
    $('#infotickerTextColor').val($(this).val());
    formChangedCallback();
  });
  $('#infotickerTextColor').change( function() {
    $('#colorPaletteForTextColor').val($(this).val());
  });
  $('#colorPaletteForBgColor').change( function() {
    $('#infotickerBgColor').val($(this).val());
    formChangedCallback();
  });
  $('#infotickerBgColor').change( function() {
    $('#colorPaletteForBgColor').val($(this).val());
  });
}

async function getCounterCorrection(occupancy_value) {
  let url = new URL('/local/storeoccupancymanager/api/correction.cgi', window.location.origin);
  let urlParams = new URLSearchParams(url.search);
  urlParams.set('occupancy', occupancy_value);
  url.search = urlParams;

  let result = await $.get(url);
  if (result.status !== 200) {
    throw new Error('Unable to load correction data from camera.');
  }
  return result.data;
}

function copyValue(target, successText) {
  let jqueryInput = $(target);
  target.select();
  document.execCommand("copy");
  jqueryInput.tooltip({title: successText});
  jqueryInput.tooltip('show');
  setTimeout(function() {
    jqueryInput.tooltip('dispose');
  }, 2000);
  return false;
}

function createApcPanel() {
  const apcPanelElement = $('#apcPanel');
  apcPanelElement.find('.card').remove();
  let navTabs = '<ul class="nav nav-tabs border-bottom-0 mr-0 ml-0" id="apcTabs">';
  let navItem = '<li class="nav-item"><a class="nav-link" id="apcItem%COUNTER%-tab" data-toggle="tab" data-target="#apcItem%COUNTER%"><div id="checkConnectionApcItem%COUNTER%" class="checkStatus checkStatusApc"></div> Counter %COUNTER%</a></li>';
  // Create tabs
  for(var i = 1; i <= MAX_COUNTER_ITEMS; i++) {
    navTabs += navItem.replace(/%COUNTER%/g, i);
  }
  navTabs += '</ul>';
  // Create content
  let tabContentSource = $('#apcPanelSource').html();
  let tabContent = '<div class="tab-content" id="apcPanelTabContent">';
  for(var i = 1; i <= MAX_COUNTER_ITEMS; i++) {
    tabContent += tabContentSource.replace(/%COUNTER%/g, i);
  }
  tabContent += '</div>';
  let card = '<div id="apcPanelInputs" class="card mt-2 mb-2 ml-3 mr-3"><div class="card-header pb-0">' + navTabs + '</div><div class="card-body">' + tabContent + '</div></div>';
  apcPanelElement.append(card);
  $('#apcTabs li:first-child a').tab('show');
}

function fillApcPanel() {
  for (var i = 1; i <= MAX_COUNTER_ITEMS; i++) {
    let apcItem = serviceArrTmp.counter_camera_list[i-1];
    $('#apcItem' + i + 'CameraHost').val(apcItem.host);
    $('#apcItem' + i + 'CameraPort').val(apcItem.port);
    $('#apcItem' + i + 'CameraUser').val(apcItem.user);
    $('#apcItem' + i + 'CameraPass').val(apcItem.pass);
    $('#apcItem' + i + 'CameraElementName').val(apcItem.element_name);
  }
}

function createAdditionalGraphicsPanel() {
  const additionalGraphicsPanelElement = $('#additionalGraphicsPanel');
  additionalGraphicsPanelElement.find('.card').remove();
  let navTabs = '<ul class="nav nav-tabs border-bottom-0 mr-0 ml-0" id="additionalGraphicsTabs">';
  let navItem = '<li class="nav-item"><a class="nav-link" id="#image%COUNTER%-tab" data-toggle="tab" data-target="#image%COUNTER%">Image %COUNTER%</a></li>';
  // Create tabs
  for(var i = 1; i <= MAX_ADDITIONAL_ITEMS; i++) {
    navTabs += navItem.replace(/%COUNTER%/g, i);
  }
  navTabs += '</ul>';

  // Create content
  let tabContentSource = $('#additionalGraphicsSource').html();
  let tabContent = '<div class="tab-content" id="additionalGraphicsTabContent">';
  for(var i = 1; i <= MAX_ADDITIONAL_ITEMS; i++) {
    tabContent += tabContentSource.replace(/%COUNTER%/g, i);
  }
  tabContent += '</div>';
  let card = '<div class="card"><div class="card-header pb-0">' + navTabs + '</div><div class="card-body">' + tabContent + '</div></div>';
  additionalGraphicsPanelElement.append(card);
  $('#additionalGraphicsTabs li:first-child a').tab('show');
  additionalGraphicsPanelElement.hideSpinner();
  $('#additionalGraphicsPanel [title]').tooltip();
}

function fillAdditionalGraphicsPanel() {
  for (var i = 1; i <= MAX_ADDITIONAL_ITEMS; i++) {
    let additionalGraphicsImageElementId = '#additionalGraphicsImage' + i;
    let additionalGraphicsVisibilityElementId = '#additionalGraphicsVisibility' + i;
    let additionalGraphicsPositionElementId = '#additionalGraphicsPosition' + i;
    let additionalGraphicsOffsetXElementId = '#additionalGraphicsOffsetX' + i;
    let additionalGraphicsOffsetYElementId = '#additionalGraphicsOffsetY' + i;
    let additionalGraphicsScaleElementId = '#additionalGraphicsScale' + i;
    let additionalGraphicsItem = serviceArrTmp.additional_graphics[i-1];

    createImageListInOption(additionalGraphicsImageElementId, true, 'None', additionalGraphicsItem.image);
    createVisibilityList(additionalGraphicsVisibilityElementId, additionalGraphicsItem.visibility);
    createPositionList(additionalGraphicsPositionElementId, additionalGraphicsItem.coordSystem);
    $(additionalGraphicsOffsetXElementId).val(additionalGraphicsItem.pos_x);
    $(additionalGraphicsOffsetYElementId).val(additionalGraphicsItem.pos_y);
    $(additionalGraphicsScaleElementId).val(convertScaleFloatToPercentage(additionalGraphicsItem.scale));
  }
  $('#additionalGraphicsPanel .form-control').unbind().change(formChangedCallback);
  setAdditionalGraphicsOverlayStatus();
}

function getCameras() {
  let counterCameraTypeVal = $('#counterCameraType').val();
  let camerasArr = [];
  if (counterCameraTypeVal === 'aoe' || counterCameraTypeVal === 'axis3d') {
    camerasArr.push(
      {
        statusElementId: null,
        cameraHostVal: $('#counterCameraHost').val(),
        cameraPortVal: $('#counterCameraPort').val(),
        cameraUserVal: $('#counterCameraUser').val(),
        cameraPassVal: $('#counterCameraPass').val(),
        cameraTypeVal: $('#counterCameraType').val(),
      }
    );
  } else {
    for (var i = 1; i <= MAX_COUNTER_ITEMS; i++) {
      camerasArr.push(
        {
          statusElementId: '#checkConnectionApcItem' + i,
          cameraHostVal: $('#apcItem' + i + 'CameraHost').val(),
          cameraPortVal: $('#apcItem' + i + 'CameraPort').val(),
          cameraUserVal: $('#apcItem' + i + 'CameraUser').val(),
          cameraPassVal: $('#apcItem' + i + 'CameraPass').val(),
          cameraTypeVal: $('#counterCameraType').val(),
        }
      );
    }
  }
  return camerasArr;
}

function checkConnection() {
  var promise = new Promise(async function(resolve, reject) {

    let arrCameras = getCameras();
    let results = [];

    for (var i = 0; i < arrCameras.length; i++) {
      const resultMsg = await getConnection(arrCameras[i]);
      if (arrCameras[i]['statusElementId'] !== null) {
        setCheckConnectionElement(arrCameras[i]['statusElementId'], resultMsg);
      }
      results.push(resultMsg);
    }

    const finalResult = getAggregatedConnectionCheckResult(results);
    setCheckConnectionElement('#checkConnection', finalResult);

    resolve(!results.includes('ERR_CONNECTION'));
  });
  return promise;
}

/**
 * @param {Array} results
 * @return {String}
 */
function getAggregatedConnectionCheckResult(results) {
  const states_by_priority = [
    'ERR_CONNECTION',
    'OK',
    'ERR_EMPTY_CRE',
  ];

  const states_in_result = states_by_priority.filter(item => results.includes(item));
  states_in_result.push('OK'); //add default state

  return states_in_result.shift();
}

async function getConnection(camera) {
  return await new Promise(async function(resolve, reject) {
    let counterCameraHost = camera['cameraHostVal'];
    let counterCameraPort = camera['cameraPortVal'];
    let counterCameraUser = camera['cameraUserVal'];
    let counterCameraPass = camera['cameraPassVal'];
    let counterCameraType = camera['cameraTypeVal'];
    if (counterCameraHost !== '' && counterCameraHost.indexOf("http") === -1 && counterCameraPort !== ''
      && counterCameraUser !== '' && counterCameraPass !== '') {
      const connectionResult = await checkProxyCameraConnection(counterCameraHost, counterCameraPort, counterCameraUser, counterCameraPass, counterCameraType);
      if (connectionResult[1] === 'success') {
        resolve('OK');
      } else {
        resolve('ERR_CONNECTION');
      }
    } else {
      resolve('ERR_EMPTY_CRE');
    }
  });
}

function setAdditionalGraphicsOverlayStatus() {
  let enabledAdditionalGraphicsOverlayCount = enabledAdditionalGraphicsAlwaysCount() + enabledAdditionalGraphicsNoEntry() + enabledAdditionalGraphicsEntryAllowed();
  if(enabledAdditionalGraphicsOverlayCount > 0) {
    setServiceStatus('#checkAdditionalGraphicsOverlayStatus', "Yes");
  } else {
    setServiceStatus('#checkAdditionalGraphicsOverlayStatus', "No");
  }
}

function setServiceStatus(serviceStatusElementId, enabled) {
  let tooltipText = enabled === "Yes" ? 'Enabled.' : 'Disabled.';
  let status = enabled === "Yes" ? 'success' : 'disabled';
  $(serviceStatusElementId)
    .tooltip('dispose')
    .removeClass('success disabled')
    .addClass(status)
    .show().tooltip({title: tooltipText});
}

function setCheckConnectionElement(connectionElementId, resultMsg) {
  const msgMap = {
    'OK': {
      msg: 'Connection successful.',
      status: 'success',
    },
    'ERR_CONNECTION': {
      msg: 'Connection error.',
      status: 'danger',
    },
    'ERR_EMPTY_CRE': {
      msg: 'Please fill counter credentials.',
      status: '',
    },
  };
  $(connectionElementId)
    .tooltip('dispose')
    .removeClass('danger success')
    .addClass(msgMap[resultMsg]['status'])
    .show().tooltip({title: msgMap[resultMsg]['msg']});
}

function saveAdditionalGraphicsPanel() {
  for (var i = 1; i <= MAX_ADDITIONAL_ITEMS; i++) {
    let additionalGraphicsImageElementId = '#additionalGraphicsImage' + i;
    let additionalGraphicsVisibilityElementId = '#additionalGraphicsVisibility' + i;
    let additionalGraphicsPositionElementId = '#additionalGraphicsPosition' + i;
    let additionalGraphicsOffsetXElementId = '#additionalGraphicsOffsetX' + i;
    let additionalGraphicsOffsetYElementId = '#additionalGraphicsOffsetY' + i;
    let additionalGraphicsScaleElementId = '#additionalGraphicsScale' + i;

    serviceArrTmp.additional_graphics[i-1].image = $(additionalGraphicsImageElementId).val();
    serviceArrTmp.additional_graphics[i-1].visibility = $(additionalGraphicsVisibilityElementId).val();
    serviceArrTmp.additional_graphics[i-1].coordSystem = $(additionalGraphicsPositionElementId).val();
    serviceArrTmp.additional_graphics[i-1].pos_x = parseInt($(additionalGraphicsOffsetXElementId).val());
    serviceArrTmp.additional_graphics[i-1].pos_y = parseInt($(additionalGraphicsOffsetYElementId).val());
    serviceArrTmp.additional_graphics[i-1].scale = convertScalePercentageToFloat($(additionalGraphicsScaleElementId).val());
  }
}

function saveApcPanel() {
  for (var i = 1; i <= MAX_COUNTER_ITEMS; i++) {
    let apcCameraHostId = '#apcItem' + i + 'CameraHost';
    let apcCameraPortId = '#apcItem' + i + 'CameraPort';
    let apcCameraUserId = '#apcItem' + i + 'CameraUser';
    let apcCameraPassId = '#apcItem' + i + 'CameraPass';
    let apcCameraElementNameId = '#apcItem' + i + 'CameraElementName';

    serviceArrTmp.counter_camera_list[i-1].host = $(apcCameraHostId).val();
    serviceArrTmp.counter_camera_list[i-1].port = parseInt($(apcCameraPortId).val());
    serviceArrTmp.counter_camera_list[i-1].user = $(apcCameraUserId).val();
    serviceArrTmp.counter_camera_list[i-1].pass = $(apcCameraPassId).val();
    serviceArrTmp.counter_camera_list[i-1].element_name = $(apcCameraElementNameId).val();
  }
}

async function refreshSelectWithImageOptions() {
  let counterGraphicsImageEnter = '#counterGraphicsImageEnter';
  createImageListInOption(counterGraphicsImageEnter, true, 'Default', serviceArrTmp.counter_graphics.image_enter);
  let counterGraphicsImageStopId = '#counterGraphicsImageStop';
  createImageListInOption(counterGraphicsImageStopId, true, 'Default', serviceArrTmp.counter_graphics.image_stop);
  for(var i = 1; i <= MAX_ADDITIONAL_ITEMS; i++) {
    let additionalGraphicsImageElementId = '#additionalGraphicsImage' + i;
    let additionalGraphicsItem = serviceArrTmp.additional_graphics[i - 1];
    createImageListInOption(additionalGraphicsImageElementId, true, 'None', additionalGraphicsItem.image);
  }
}

function saveCameraList() {
  let cameraItems = $('.videoChannel');
  let cameraList = [];
  for(var i = 0; i < cameraItems.length; i++) {
    if(cameraItems[i].checked) {
      cameraList.push(i);
    }
  }
  return cameraList;
}

function formChangedCallback() {
  appEnabled = $('#enabled').val() === 'Yes' ? 1 : 0;
  serviceArrTmp.stream_width = cameraResolutions[parseInt($('#resolution').val())].width;
  serviceArrTmp.stream_height = cameraResolutions[parseInt($('#resolution').val())].height;
  serviceArrTmp.max_in_store_limit = parseInt($('#maxInStoreLimit').val());
  serviceArrTmp.camera_list = saveCameraList();
  serviceArrTmp.output.type = $('#outputType').val();
  counterCorrection.reset_time = $('#counterCorrectionReset').val();
  serviceArrTmp.counter_camera_type = $('#counterCameraType').val();
  serviceArrTmp.counter_camera_host = $('#counterCameraHost').val();
  serviceArrTmp.counter_camera_port = parseInt($('#counterCameraPort').val());
  serviceArrTmp.counter_camera_user = $('#counterCameraUser').val();
  serviceArrTmp.counter_camera_pass = $('#counterCameraPass').val();
  saveApcPanel();

  saveCounterGraphicsSettings();
  saveAdditionalGraphicsPanel();
  saveInfotickerSettings();

  createCameraPreview($('#videoChannel'), serviceArrTmp.stream_width, serviceArrTmp.stream_height, serviceArrTmp.camera_list);
  showSaveWarning();
  checkGraphicsCount();
}

function saveCounterGraphicsSettings() {
  serviceArrTmp.counter_graphics.enabled = $('#counterGraphicsEnabled').val() === 'Yes';
  serviceArrTmp.counter_graphics.coordSystem = $('#counterGraphicsPosition').val();
  serviceArrTmp.counter_graphics.pos_x = parseInt($('#counterGraphicsOffsetX').val());
  serviceArrTmp.counter_graphics.pos_y = parseInt($('#counterGraphicsOffsetY').val());
  serviceArrTmp.counter_graphics.scale = convertScalePercentageToFloat($('#counterGraphicsScale').val());
  serviceArrTmp.counter_graphics.image_enter = $('#counterGraphicsImageEnter').val();
  serviceArrTmp.counter_graphics.image_stop = $('#counterGraphicsImageStop').val();
  serviceArrTmp.counter_graphics.label_max_limit = $('#counterGraphicsLabelMaxLimit').val();
  serviceArrTmp.counter_graphics.label_in_store = $('#counterGraphicsLabelInStore').val();
  serviceArrTmp.counter_graphics.label_may_enter = $('#counterGraphicsLabelMayEnter').val();
  if (!isMips()) {
    serviceArrTmp.counter_graphics.fullscreen = $('#counterGraphicsFullscreen').val() === 'Yes';
  } else {
    serviceArrTmp.counter_graphics.fullscreen = false;
  }
}

function saveInfotickerSettings() {
  serviceArrTmp.infoticker.enabled = $('#infotickerEnabled').val() === 'Yes';
  serviceArrTmp.infoticker.showClock = $('#infotickerShowClock').val() === 'Yes' ? 1 : 0;
  serviceArrTmp.infoticker.clockType = $('#infotickerRadioClock').val();
  serviceArrTmp.infoticker.textColor = hexToRgb($('#infotickerTextColor').val());
  serviceArrTmp.infoticker.bgColor =  hexToRgb($('#infotickerBgColor').val());
  serviceArrTmp.infoticker.sourceType = $('#infotickerSourceType').val();
  serviceArrTmp.infoticker.fontSize = parseInt($('#infotickerFontSize').val());
  serviceArrTmp.infoticker.source = $('#infotickerSource').val();
  serviceArrTmp.infoticker.numberOfLines = parseInt($('#infotickerNumberOfLines').val());
  serviceArrTmp.infoticker.switchingTime = parseInt($('#infotickerSwitchingTime').val());
  let crawlLeftValue = $('#infotickerCrawlLeft').val();
  if(!isMips()) {
    if (crawlLeftValue === "No") {
      serviceArrTmp.infoticker.crawlLeft =  false;
      serviceArrTmp.infoticker.crawlSpeed = 0;
    } else {
      serviceArrTmp.infoticker.crawlLeft =  true;
      if(crawlLeftValue === "Slow") serviceArrTmp.infoticker.crawlSpeed = 1;
      if(crawlLeftValue === "Medium") serviceArrTmp.infoticker.crawlSpeed = 2;
      if(crawlLeftValue === "Fast") serviceArrTmp.infoticker.crawlSpeed = 4;
    }
  }
}

function saveBtnCallback(silence, enabled_param, services_param, correction_param) {

  services_param = services_param || serviceArrTmp;
  serviceArr = JSON.parse(JSON.stringify(services_param));// Set from temporary to permanent arr

  enabled_param = enabled_param || appEnabled;

  correction_param = correction_param || counterCorrection;

  // Save params to camera
  setService(
    String(enabled_param),
    JSON.stringify(serviceArr),
    JSON.stringify(correction_param)
  ).then(function() {
    hideSaveWarning();
    if (silence !== true)
      showMessage('success', 'Parameters have been successfully updated.', false, 'successParameters');
  }, function() {
    if (silence !== true)
      showMessage('danger', 'Parameters setting error.', false, 'errorParameters');
  });
}

async function discardBtnCallback() {
  $('#counterCorrectionOccupancy').val("");
  serviceArrTmp = Object.assign(createDefaultServiceArr(), JSON.parse(serviceArr));
  counterCorrection = Object.assign(DEFAULT_COUNTER_CORRECTION, JSON.parse(params[appNameLowered + '.countercorrection'] || '{}'));
  await initForm();
  registerEvents();
  const connectionResult = await checkConnection();
  collapseCounterGraphicsSettingsContent(connectionResult);
  checkGraphicsCount();
}

async function getLatestACAPVersion(appName, cameraModel, serverUrl) {
  return await new Promise(function(resolve, reject) {
    $.get({
      url: serverUrl + '/index.php?p=ACAPPackage&a=getPackageInfo',
      data: {
        app_name: appName,
        camera_model: cameraModel
      },
      success: function(response) {
        resolve(response.version);
      },
      error: function() {
        reject();
      }
    });
  });
}

function showImageFromCamera(width, height, videoChannel, elementId) {
  var resolution = '';
  var camera = '';
  if (width !== undefined && height !== undefined && width !== '' && height !== '') {
    resolution = '&resolution=' + width + 'x' + height;
  }
  if (videoChannel !== undefined && videoChannel !== null && videoChannel > -1) {
    camera = '&camera=' + (videoChannel + 1);
  }
  if (params['image.i'  + videoChannel + '.source'] === 'quad') {
    camera = '&camera=quad';
  }
  if (!elementId) {
    elementId = 'imagePreviewCamera';
  }
  $('#' + elementId).attr("src", cameraImgUrl + resolution + camera + '&t=' + Date.now());
}

function isArmv7hf() {
 return cameraArchitecture === 'armv7hf';
}

function isMips() {
  return cameraArchitecture === 'mips';
}

function showLicenseInfoCallback() {
  let state = 'No license';
  let expiration = licenseExpiration;
  if (licenseState !== "Missing") {
    state = licenseState;
  }
  if (licenseActivated && licenseExpiration === '') {
    expiration = 'Unlimited';
  }
  licenseInfo = '<dl class="dl-horizontal">';
  licenseInfo += '<dt>Status</dt><dd>' + state + '</dd>';
  if(licenseState === 'Valid' || licenseState === 'Invalid') {
    licenseInfo += '<dt>Expiration</dt><dd>' + expiration + '</dd>';
  }
  licenseInfo += '</dl>';

  $('#licenseInfoBody').html(licenseInfo);
  $('#licenseInfoModal').modal('show');
}

function showExpiredLicense() {
  $('.content').hide();
  $('#imageManagerBtn').hide();
  $('.expired_license').show();
}

function getPreviewImage(request) {
  previewRequests.push(request);
  if (previewRequests.length == 1) {
    $.get(request, function(response) {
      if (serviceArrTmp.enabled) {
        var img = '<img id="imgPreview" style="display: none" src="data:image/png;base64,' + response.data + '">';
        $("#imagePreviews").html(img);
        setPreviewPos(response.pos_x, response.pos_y, response.width, response.height, response.stream_width, response.stream_height);
      }
      previewRequests.shift();
      var newReq = previewRequests.pop();
      previewRequests = [];
      if (newReq != undefined)
        getPreviewImage(newReq);
    });
  }
}

async function showImageManagerCallback() {
  await imageManager.init();
  imageManager.open();
}

function startAcapApp() {
  var promise = new Promise(function(resolve, reject) {
    $.get('/axis-cgi/applications/control.cgi?package=' + appNameLowered + '&action=start', function(response) {
      response = response.trim().toLowerCase();

      var errorCode = '';
      var found = response.search('error:');
      if (found != -1)
        errorCode = response.substring(7);

      if (response == 'ok') {
        setTimeout(resolve, 2000);  // Wait for acap start
      } else if (errorCode == 6) {
        resolve(); // already running
      } else {
        showMessage('danger', 'Could not start camoverlay package, error: ' + errorCode + '.');
        resolve();
      }
    });
  });
  return promise;
}

function showMessage(type, html, static, msgName) {
  var header = 'Info:';
  if (type == 'success')
    header = 'Success:';
  else if (type == 'warning')
    header = 'Warning!';
  else if (type == 'danger')
    header = 'Error!';

  var id = alertID++;
  var exists = 0;
  var targetDiv = (static === false || static === undefined) ? '#messages_fluid' : '#messages';

  if ($(targetDiv).find('.' + msgName).length > 0) exists = 1;
  if (exists === 0 || msgName === undefined) {
    if (msgName === undefined) msgName = "";
    var message = '<div id="alert' + id + '" class="alert alert-dismissible alert-' + type + ' ' + msgName + '">';
    if(static !== false) { message += '<button type="button" class="close" data-dismiss="alert">&times;</button>'; }
    message += '<strong class="mr-2">' + header + '</strong>';
    message += html + '</div>';
    $(targetDiv).append(message);
  }

  if (static === undefined || static === false) {
    $('#alert' + id).fadeTo(5000, 500).slideUp(500, function(){
      $('#alert' + id).alert('close');
    });
  }
}
